@@ -26,13 +26,14 @@
 # installed, because pkg_resources is necessary to read eggs.
 
 from django.core.exceptions import ImproperlyConfigured
-from django.template.base import Origin, Template, Context, TemplateDoesNotExist, add_to_builtins
+from django.template.base import Origin, Template, Context, TemplateDoesNotExist
 from django.conf import settings
-from django.utils.module_loading import import_by_path
+from django.utils.module_loading import import_string
 from django.utils import six
 
 template_source_loaders = None
 
+
 class BaseLoader(object):
     is_usable = False
 
@@ -61,7 +62,7 @@ class BaseLoader(object):
         name.
 
         """
-        raise NotImplementedError
+        raise NotImplementedError('subclasses of BaseLoader must provide a load_template_source() method')
 
     def reset(self):
         """
@@ -71,6 +72,7 @@ class BaseLoader(object):
         """
         pass
 
+
 class LoaderOrigin(Origin):
     def __init__(self, display_name, loader, name, dirs):
         super(LoaderOrigin, self).__init__(display_name)
@@ -79,19 +81,21 @@ class LoaderOrigin(Origin):
     def reload(self):
         return self.loader(self.loadname, self.dirs)[0]
 
+
 def make_origin(display_name, loader, name, dirs):
     if settings.TEMPLATE_DEBUG and display_name:
         return LoaderOrigin(display_name, loader, name, dirs)
     else:
         return None
 
+
 def find_template_loader(loader):
     if isinstance(loader, (tuple, list)):
         loader, args = loader[0], loader[1:]
     else:
         args = []
     if isinstance(loader, six.string_types):
-        TemplateLoader = import_by_path(loader)
+        TemplateLoader = import_string(loader)
 
         if hasattr(TemplateLoader, 'load_template_source'):
             func = TemplateLoader(*args)
@@ -110,6 +114,7 @@ def find_template_loader(loader):
     else:
         raise ImproperlyConfigured('Loader does not define a "load_template" callable template source loader')
 
+
 def find_template(name, dirs=None):
     # Calculate template_source_loaders the first time the function is executed
     # because putting this logic in the module-level namespace may cause
@@ -130,17 +135,19 @@ def find_template(name, dirs=None):
             pass
     raise TemplateDoesNotExist(name)
 
-def get_template(template_name):
+
+def get_template(template_name, dirs=None):
     """
     Returns a compiled Template object for the given template name,
     handling template inheritance recursively.
     """
-    template, origin = find_template(template_name)
+    template, origin = find_template(template_name, dirs)
     if not hasattr(template, 'render'):
         # template needs to be compiled
         template = get_template_from_string(template, origin, template_name)
     return template
 
+
 def get_template_from_string(source, origin=None, name=None):
     """
     Returns a compiled Template object for the given template code,
@@ -148,41 +155,40 @@ def get_template_from_string(source, origin=None, name=None):
     """
     return Template(source, origin, name)
 
-def render_to_string(template_name, dictionary=None, context_instance=None):
+
+def render_to_string(template_name, dictionary=None, context_instance=None,
+                     dirs=None):
     """
     Loads the given template_name and renders it with the given dictionary as
     context. The template_name may be a string to load a single template using
     get_template, or it may be a tuple to use select_template to find one of
     the templates in the list. Returns a string.
     """
-    dictionary = dictionary or {}
     if isinstance(template_name, (list, tuple)):
-        t = select_template(template_name)
+        t = select_template(template_name, dirs)
     else:
-        t = get_template(template_name)
+        t = get_template(template_name, dirs)
     if not context_instance:
         return t.render(Context(dictionary))
+    if not dictionary:
+        return t.render(context_instance)
     # Add the dictionary to the context stack, ensuring it gets removed again
     # to keep the context_instance in the same state it started in.
-    context_instance.update(dictionary)
-    try:
+    with context_instance.push(dictionary):
         return t.render(context_instance)
-    finally:
-        context_instance.pop()
 
-def select_template(template_name_list):
+
+def select_template(template_name_list, dirs=None):
     "Given a list of template names, returns the first that can be loaded."
     if not template_name_list:
         raise TemplateDoesNotExist("No template names provided")
     not_found = []
     for template_name in template_name_list:
         try:
-            return get_template(template_name)
+            return get_template(template_name, dirs)
         except TemplateDoesNotExist as e:
             if e.args[0] not in not_found:
                 not_found.append(e.args[0])
             continue
     # If we get here, none of the templates could be loaded
     raise TemplateDoesNotExist(', '.join(not_found))
-
-add_to_builtins('django.template.loader_tags')
