@@ -1,12 +1,14 @@
-from __future__ import absolute_import, unicode_literals
+from __future__ import unicode_literals
 
 from datetime import datetime
 from operator import attrgetter
+from unittest import skipUnless
 
 from django.core.exceptions import FieldError
-from django.test import TestCase, skipUnlessDBFeature
+from django.db import connection
+from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
-from .models import Author, Article, Tag, Game, Season, Player
+from .models import Author, Article, Tag, Game, Season, Player, MyISAMArticle
 
 
 class LookupTests(TestCase):
@@ -125,7 +127,7 @@ class LookupTests(TestCase):
     def test_values(self):
         # values() returns a list of dictionaries instead of object instances --
         # and you can specify which fields you want to retrieve.
-        identity = lambda x:x
+        identity = lambda x: x
         self.assertQuerysetEqual(Article.objects.values('headline'),
             [
                 {'headline': 'Article 5'},
@@ -256,7 +258,7 @@ class LookupTests(TestCase):
         # returned as a list of tuples, rather than a list of dictionaries.
         # Within each tuple, the order of the elements is the same as the order
         # of fields in the values_list() call.
-        identity = lambda x:x
+        identity = lambda x: x
         self.assertQuerysetEqual(Article.objects.values_list('headline'),
             [
                 ('Article 5',),
@@ -283,26 +285,26 @@ class LookupTests(TestCase):
             Article.objects.extra(select={'id_plus_one': 'id+1'})
                            .order_by('id').values_list('id_plus_one', 'id'),
             [
-                (self.a1.id+1, self.a1.id),
-                (self.a2.id+1, self.a2.id),
-                (self.a3.id+1, self.a3.id),
-                (self.a4.id+1, self.a4.id),
-                (self.a5.id+1, self.a5.id),
-                (self.a6.id+1, self.a6.id),
-                (self.a7.id+1, self.a7.id)
+                (self.a1.id + 1, self.a1.id),
+                (self.a2.id + 1, self.a2.id),
+                (self.a3.id + 1, self.a3.id),
+                (self.a4.id + 1, self.a4.id),
+                (self.a5.id + 1, self.a5.id),
+                (self.a6.id + 1, self.a6.id),
+                (self.a7.id + 1, self.a7.id)
             ],
             transform=identity)
         self.assertQuerysetEqual(
             Article.objects.extra(select={'id_plus_one': 'id+1'})
                            .order_by('id').values_list('id', 'id_plus_one'),
             [
-                (self.a1.id, self.a1.id+1),
-                (self.a2.id, self.a2.id+1),
-                (self.a3.id, self.a3.id+1),
-                (self.a4.id, self.a4.id+1),
-                (self.a5.id, self.a5.id+1),
-                (self.a6.id, self.a6.id+1),
-                (self.a7.id, self.a7.id+1)
+                (self.a1.id, self.a1.id + 1),
+                (self.a2.id, self.a2.id + 1),
+                (self.a3.id, self.a3.id + 1),
+                (self.a4.id, self.a4.id + 1),
+                (self.a5.id, self.a5.id + 1),
+                (self.a6.id, self.a6.id + 1),
+                (self.a7.id, self.a7.id + 1)
             ],
             transform=identity)
         self.assertQuerysetEqual(
@@ -394,9 +396,9 @@ class LookupTests(TestCase):
                                  ['<Article: Article with \ backslash>'])
 
     def test_exclude(self):
-        a8 = Article.objects.create(headline='Article_ with underscore', pub_date=datetime(2005, 11, 20))
-        a9 = Article.objects.create(headline='Article% with percent sign', pub_date=datetime(2005, 11, 21))
-        a10 = Article.objects.create(headline='Article with \\ backslash', pub_date=datetime(2005, 11, 22))
+        Article.objects.create(headline='Article_ with underscore', pub_date=datetime(2005, 11, 20))
+        Article.objects.create(headline='Article% with percent sign', pub_date=datetime(2005, 11, 21))
+        Article.objects.create(headline='Article with \\ backslash', pub_date=datetime(2005, 11, 22))
 
         # exclude() is the opposite of filter() when doing lookups:
         self.assertQuerysetEqual(
@@ -436,7 +438,7 @@ class LookupTests(TestCase):
             ])
 
     def test_none(self):
-       # none() returns a QuerySet that behaves like any other QuerySet object
+        # none() returns a QuerySet that behaves like any other QuerySet object
         self.assertQuerysetEqual(Article.objects.none(), [])
         self.assertQuerysetEqual(
             Article.objects.none().filter(headline__startswith='Article'), [])
@@ -470,13 +472,15 @@ class LookupTests(TestCase):
             self.fail('FieldError not raised')
         except FieldError as ex:
             self.assertEqual(str(ex), "Cannot resolve keyword 'pub_date_year' "
-                             "into field. Choices are: author, headline, id, pub_date, tag")
+                             "into field. Choices are: author, author_id, headline, "
+                             "id, pub_date, tag")
         try:
             Article.objects.filter(headline__starts='Article')
             self.fail('FieldError not raised')
         except FieldError as ex:
-            self.assertEqual(str(ex), "Join on field 'headline' not permitted. "
-                             "Did you misspell 'starts' for the lookup type?")
+            self.assertEqual(
+                str(ex), "Unsupported lookup 'starts' for CharField "
+                "or join on the field not permitted.")
 
     def test_regex(self):
         # Create some articles with a bit more interesting headlines for testing field lookups:
@@ -627,7 +631,7 @@ class LookupTests(TestCase):
 
     def test_regex_non_ascii(self):
         """
-        Ensure that a regex lookup does not trip on non-ascii characters.
+        Ensure that a regex lookup does not trip on non-ASCII characters.
         """
         Player.objects.create(name='\u2660')
         Player.objects.get(name__regex='\u2660')
@@ -708,3 +712,27 @@ class LookupTests(TestCase):
         self.assertEqual(Player.objects.filter(games__season__gt=333).distinct().count(), 2)
         self.assertEqual(Player.objects.filter(games__season__year__gt=2010).distinct().count(), 2)
         self.assertEqual(Player.objects.filter(games__season__gt__gt=222).distinct().count(), 2)
+
+
+class LookupTransactionTests(TransactionTestCase):
+    available_apps = ['lookup']
+
+    @skipUnless(connection.vendor == 'mysql', 'requires MySQL')
+    def test_mysql_lookup_search(self):
+        # To use fulltext indexes on MySQL either version 5.6 is needed, or one must use
+        # MyISAM tables. Neither of these combinations is currently available on CI, so
+        # lets manually create a MyISAM table for Article model.
+        with connection.cursor() as cursor:
+            cursor.execute(
+                "CREATE TEMPORARY TABLE myisam_article ("
+                "    id INTEGER PRIMARY KEY AUTO_INCREMENT, "
+                "    headline VARCHAR(100) NOT NULL "
+                ") ENGINE MYISAM")
+            dr = MyISAMArticle.objects.create(headline='Django Reinhardt')
+            MyISAMArticle.objects.create(headline='Ringo Star')
+            # NOTE: Needs to be created after the article has been saved.
+            cursor.execute(
+                'CREATE FULLTEXT INDEX myisam_article_ft ON myisam_article (headline)')
+            self.assertQuerysetEqual(
+                MyISAMArticle.objects.filter(headline__search='Reinhardt'),
+                [dr], lambda x: x)
